{% ckan_extends %}

{% block styles %}
    {{ super() }}
    <link rel="stylesheet" href="https://cdn.jsdelivr.net/npm/choices.js@9.0.1/public/assets/styles/choices.min.css" />
    <style>
        .bg-light {
            background: #eef0f4;
        }

        .choices__list--dropdown .choices__item--selectable {
            padding-right: 1rem;
        }

        .choices__list--single {
            padding: 0;
        }

        .card {
            transform: translateY(-50%);
        }

        .choices[data-type*=select-one]:after {
            right: 1.5rem;
        }

        .shadow {
            box-shadow: 0.3rem 0.3rem 1rem rgba(178, 200, 244, 0.23);
        }

        a {
            text-decoration: none;
            color: inherit;
            transition: all 0.3s;
        }

    </style>
{% endblock %}